/** @license
 * Copyright (c) 2010-2013 iNTERFACEWARE Inc.  All rights reserved.
 */
 
var DEMOmanager = function() {
   // Set this to true after prompting the user once to avoid re-prompting them
   // every time new dashboard data comes in.
   var m_HasPrompted = false;
   var s_DemoNoReminderKey = "DemoNoReminder";
   
   // Returns true if the Iguana installation was pre-existing, false otherwise.
   // We perform this check by analzying the response from a /dashboard_data
   // request.
   function isOldInstall(DashboardData) {
      var Servers = DashboardData.Servers;
      // If the Servers array doesn't exist then this likely means that
      // include_remote_servers was set to false when the request was sent.
      // Not sure under what circumstances this would happen, but it's good
      // practice to check for it anyway.
      if (Servers) {
         // If the global dashboard has been configured then this definitely
         // isn't a new installation.
         if (Servers.length > 1) {
            return true;
         }
         
         var LocalData = Servers[0].DashboardData;
         if (LocalData && LocalData.TotalCountOfChannel > 0) {
            return true;
         }
      }
      
      return false;
   }
   
   // Public functions.
   var Manager = {};
   
   // We expose this function so that users can force the demo dialog to appear
   // from the Settings page.
   Manager.openPopup = function(UserInitiated) {
      $("body").append("\
         <div id='demo-popup'>\
            <p> <strong> Welcome to Iguana! </strong> For all first-time users, we offer a quick demo and four sample interfaces. Would you like us to build these demo channels for you? </p>\
            <p> <button id='demo-start'> Yes, please run the demo! </button> </p>\
            <p>\
               <button id='demo-decline'> No thanks" + (UserInitiated ? "." : ", I just want to start using Iguana.") + "</button>" +
               (UserInitiated ? "" : "<input type='checkbox' id='demo-no-reminder' style='margin-left: 20px;'> <label for='demo-no-reminder'> Don't ask me again. </label>") +
            "</p>" +
            // We add this image to the dialog right away so that we can be
            // sure it's loaded before making the Ajax call to set up the demo
            // channels. Otherwise it won't show up in the dialog unless it's
            // been cached already.
            "<img src='jqueryFileTree/images/spinner.gif' style='display: none;'>\
         </div>\
      ");
      var DemoPopup = $("#demo-popup");
      
      // Helper function for displaying the final message in the dialog.
      function finalizeDisplay(Content, ErrorOccurred) {
         DemoPopup.html(Content);
         
         if (ErrorOccurred) {
            if (!UserInitiated) {
               DemoPopup.append("<p> Ready to try again? Refresh this page OR open\
                     <a href='advanced.html'> Settings</a> and click &quot;Load Demo Channels&quot;. </p>");
            }
            DemoPopup.append("<p> If you are having trouble resolving this error, please\
                  <a href=mailto:support@interfaceware.com?Subject=Iguana%20Channel%20Demo%20Failed> email us</a>. </p>");
         }
         
         DemoPopup.dialog("option", "buttons", {
            Close: function() {
               $(this).dialog("close");
            }
         });
      }
      
      $("#demo-start").button().click(function() {
         DemoPopup.html("\
            <p> Loading demo channels... <img src='jqueryFileTree/images/spinner.gif'> </p>\
         ");
         
         // No params needed. User validation should happen automatically in
         // DWBrequestHandler.
         AJAXpost("/setup_demo", "",
            function(ResponseText) { // success handler
               // Check for an error message in the response.
               try {
                  var ErrorMsg;
                  // Some JSON parsers treat an empty string as an error.
                  if (ResponseText != "") {
                     // DWB surrounds JSON in parentheses for some reason. These
                     // should be removed before attempting to parse the
                     // response as JSON.
                     if (ResponseText[0] == "(" && ResponseText[ResponseText.length - 1] == ")") {
                        ResponseText = ResponseText.substring(1, ResponseText.length - 1);
                     }
                     ErrorMsg = $.parseJSON(ResponseText).ErrorDescription; // can throw
                  }
                  
                  if (ErrorMsg) {
                     finalizeDisplay("\
                        <p> We're sorry, but we could not load the demo. The following error occurred: </p>\
                        <p>" + ErrorMsg + "</p>\
                     ", true);
                  }
                  else {
                     finalizeDisplay("\
                        <p> Success! The demo channels have loaded correctly.\
                        Now you are ready to <a href='http://www.interfaceware.com/start/' target='_blank'> learn how this demo works</a>. </p>\
                        <p> We hope you enjoy using Iguana! </p>" +
                        (UserInitiated ? "<p> <a href='dashboard.html'> View the channels.</a> </p>" : "")
                     );
                  }
               }
               catch (e) {
                  // If we got back a non-empty response then it probably means
                  // that an error occurred, but since we can't parse it into
                  // JSON we don't know for sure.
                  finalizeDisplay("\
                     <p> Internal error: the server returned malformed JSON. The demo channels may or may not have been set up successfully. </p>\
                  ", true);
               }
            },
            function(ErrorText) { // failure handler - gets used if a network level (not application level) error occurs
               finalizeDisplay("\
                  <p> We're sorry, but the request failed. The following error occurred: </p>\
                  <p>" + ErrorText + "</p>\
               ", true);
            }
         );
      });
      
      $("#demo-decline").button().click(function() {
         if (UserInitiated) {
            DemoPopup.dialog("close");
         }
         else {
            if ($("#demo-no-reminder").get(0).checked) {
               ifware.storage.set(s_DemoNoReminderKey, "true", 365);
            }
            
            finalizeDisplay("\
               <p> No problem! You can access the demo at any time, should you change your mind.\
               Simply open <a href='advanced.html'> Settings</a> and click &quot;Load Demo Channels&quot;. </p>\
            ");
         }
      });
      
      DemoPopup.dialog({
         modal: true,
         width: 350,
         resizable: false,
         title: "Demo Setup",
         close: function() {
            $(this).remove();
         }
      });
      
      // Fix for #22776 - we trigger a resize event on the window so that the
      // diaog will be selectable in IE 10.
      $(window).resize();
   }
   
   // Only opens the dialog if certain conditions are met. Should be used from
   // the Dashboard.
   Manager.openPopupWithCheck = function(DashboardData) {
      if (m_HasPrompted || isOldInstall(DashboardData) ||
            ifware.storage.get(s_DemoNoReminderKey)) {
         return;
      }
      
      m_HasPrompted = true;
      Manager.openPopup();
   }
   
   return Manager;
}();